@@ -132,6 +132,16 @@ void AptInterface::procFinished(int exitCode, QProcess::ExitStatus exitStatus)
 		} else if( output.contains("E: Unable to fetch some archives") ) {
 			iMainWindow->operationFinished(lastMode, false, "<b>Failed to fetch packages</b><br>" \
 										   "Your network connection might be down or your catalogs could be out of date");
+		} else if( output.contains("E: Couldn't find package") ) {
+			int pos = output.indexOf("E: Couldn't find package");
+			QString msg = "<b>Missing package</b><br>";
+			if( pos!=-1 )
+				msg += output.mid(pos) + "<br>";
+			msg += "Your catalogs might be out of date";
+			iMainWindow->operationFinished(lastMode, false, msg);
+		} else if( output.contains("E: Broken packages") ) {
+			iMainWindow->operationFinished(lastMode, false, "<b>Broken packages</b><br>" \
+										   "Your system has broken packages, see log for details");
 		} else {
 			iError.setNum(exitCode);
 			iError.prepend(tr("Exit code "));
