@@ -44,8 +44,14 @@ private:
 
 	int iAnim;
 	int iAnimDir;
+	int iAnimY;
+
+public slots:
+	void orientationChanged();
+
+private slots:
+	void timerEvent();
 
 };
 
 #endif // DIMMER_H
-
