@@ -31,7 +31,7 @@ class FilterSelect : public QDialog
     Q_OBJECT
 
 public:
-    explicit FilterSelect(QWidget *parent = 0);
+	explicit FilterSelect(QString title, QWidget *parent = 0);
     ~FilterSelect();
 	void setList(QStringList labels, int selected);
 	int selection();
