@@ -26,6 +26,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef MYDEF_GTK_EXISTS
+#include <gdk/gdk.h>
+#endif
+
 #include "mainwindow.h"
 #include "version.h"
 #include "ui_mainwindow.h"
@@ -35,6 +39,7 @@
 #include "dimmer.h"
 #include "repoview.h"
 #include "help.h"
+#include "settings.h"
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
@@ -52,6 +57,7 @@ MainWindow::MainWindow(QWidget *parent) :
 	iAptInterface->setPackageView(iWinPackageView);
 	iWinRepoView = new RepoView(this);
 	iWinRepoView->setAptInterface(iAptInterface);
+	iSettings = new Settings(this);
 
 	iOperation = OpNone;
 	iDimmer = new dimmer(this);
@@ -66,6 +72,8 @@ MainWindow::~MainWindow()
 	delete iWinPackageView;
 	delete iWinRepoView;
 	delete iAptInterface;
+	delete iDimmer;
+	delete iSettings;
     delete ui;
 }
 
@@ -103,7 +111,7 @@ void MainWindow::on_btnListInstalled_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpUninstall;
 
-	iWinPackageView->resetWindow();
+	//iWinPackageView->resetWindow();
 	iWinPackageView->setStatFilter( Package::PkgStatInstalled );
 
 	busyDialog(true, tr("Operation in progress"), tr("Reading package lists"));
@@ -120,7 +128,7 @@ void MainWindow::on_btnListInstallable_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpInstall;
 
-	iWinPackageView->resetWindow();
+	//iWinPackageView->resetWindow();
 	iWinPackageView->setStatFilter( Package::PkgStatNotInstalled );
 
 	if( iAptInterface->needRepoRefresh() )
@@ -144,7 +152,7 @@ void MainWindow::on_btnUpgrade_clicked()
 	if( iOperation != OpNone ) return;
 	iOperation = OpUpgrade;
 
-	iWinPackageView->resetWindow();
+	//iWinPackageView->resetWindow();
 	iWinPackageView->setStatFilter( Package::PkgStatUpgradeable );
 
 	if( iAptInterface->needRepoRefresh() ) //|| iUpgradeAutoUpdate ) // this feature is disabled on purpose
@@ -231,10 +239,12 @@ void MainWindow::operationFinished(AptInterface::aptMode lastMode_, bool success
 		{
 			iOperation = OpNone;
 			busyDialog(false);
-			iWinPackageView->hide();
+			iWinPackageView->clearSelections();
+			iWinPackageView->close();
 			ConfirmDialog d(false, this);
 			d.setText("Operation finished","Package operations were performed successfully");
 			d.exec();
+			GdkEventIconThemeReload();
 			return;
 		}
 
@@ -335,3 +345,22 @@ void MainWindow::on_actionView_log_triggered()
 	d.setText("Log", "<font size=\"-1\">" + log + "</font>" );
 	d.exec();
 }
+
+void MainWindow::on_actionOptions_triggered()
+{
+	iSettings->openWin();
+}
+
+void MainWindow::GdkEventIconThemeReload()
+{
+#ifdef MYDEF_GTK_EXISTS
+	std::cout << "Sending GDK client event" << std::endl;
+	GdkEventClient ev;
+	ev.type = GDK_CLIENT_EVENT;
+	ev.window = NULL;
+	ev.send_event = TRUE;
+	ev.message_type = gdk_atom_intern_static_string("_GTK_LOAD_ICONTHEMES");
+	ev.data_format = 32;
+	gdk_event_send_clientmessage_toall((GdkEvent*)&ev);
+#endif
+}
