@@ -51,7 +51,7 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 	QColor base = option.palette.color(QPalette::Window);
 	QColor base2 = base;
 	base.setRgb( base.red()+15, base.green()+15, base.blue()+15 );
-	base2.setRgb( base2.red()-15, base2.green()-15, base2.blue()-15 );
+	base2.setRgb( base2.red()-20, base2.green()-20, base2.blue()-20 );
 	gradientBase.setColorAt(0, base);
 	gradientBase.setColorAt(1, base2);
 
@@ -116,7 +116,7 @@ void ListItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 	}
 
 	r = option.rect;
-	r.setRight( r.right()-statusicon.width()-4-ver_w );
+	r.setRight( r.right()-statusicon.width()-4-ver_w );  //does not work as it should?
 	QFont f = painter->font();
 	f.setBold(true);
 	painter->setFont(f);
@@ -214,13 +214,21 @@ PackageView::PackageView(QWidget *parent) : QMainWindow(parent), ui(new Ui::Pack
 
 	iDimmer = new dimmer(this);
 
+	ui->searchBar->hide();
+
 	iListCoverLabel = new QLabel("No packages",ui->listWidget);
 	iListCoverLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 	iListCoverLabel->setAlignment(Qt::AlignCenter);
+
+	iKeyFilter = new KeyEventGrabber(this);
+	ui->listWidget->installEventFilter(iKeyFilter);
 }
 
 PackageView::~PackageView()
 {
+	delete iListCoverLabel;
+	delete iKeyFilter;
+	delete iDimmer;
     delete ui;
 }
 
@@ -248,72 +256,96 @@ void PackageView::openWin(MainWindow::operation op_)
 	delegate->loadIcons();
 	ui->listWidget->setItemDelegate( delegate );
 
-	QHashIterator<QString, Package*> i( *iAptInterface->packages() );
-	while (i.hasNext())
+	if( !ui->searchBar->isVisible() )
 	{
-		i.next();
-
-		QString name = i.value()->name();
-		if( i.value()->maemoDisplayName()!="" )
-			name = i.value()->maemoDisplayName();
-		QString n( name.at(0) );
-		n = n.toUpper();
-		name.replace(0,1,n);
-		bool added = false;
-
-		if( iSelectedStatFilter == Package::PkgStatNotInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+		QHashIterator<QString, Package*> i( *iAptInterface->packages() );
+		while (i.hasNext())
 		{
-			if( !i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
-				&& !i.value()->isLocal() )
+			i.next();
+
+			QString name = i.value()->name();
+			if( i.value()->maemoDisplayName()!="" )
+				name = i.value()->maemoDisplayName();
+			QString n( name.at(0) );
+			n = n.toUpper();
+			name.replace(0,1,n);
+			bool added = false;
+
+			if( iSelectedStatFilter == Package::PkgStatNotInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
 			{
-				if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
+				if( !i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
+					&& !i.value()->isLocal() )
+					{
+					if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
+						addListItem(i.value(), name);
+						added = true;
+					}
+					else if( iSelectedCatFilter!=CatFilterAllUser ) {
+						addListItem(i.value(), name);
+						added = true;
+					}
+				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
 					addListItem(i.value(), name);
 					added = true;
 				}
-				else if( iSelectedCatFilter!=CatFilterAllUser ) {
+			}
+			if( iSelectedStatFilter == Package::PkgStatInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+			{
+				if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter)) )
+				{
+					addListItem(i.value(), name);
+					added = true;
+				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
+					addListItem(i.value(), name);
+					added = true;
+				}
+			}
+			if( iSelectedStatFilter == Package::PkgStatUpgradeable || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+			{
+				if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->versionAvailable()!=i.value()->versionInstalled()
+					&& i.value()->versionAvailable()!="" && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
+					&& !i.value()->isLocal() )
+					{
+					if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
+						addListItem(i.value(), name);
+						added = true;
+					}
+					else if( iSelectedCatFilter!=CatFilterAllUser ) {
+						addListItem(i.value(), name);
+						added = true;
+					}
+				} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
 					addListItem(i.value(), name);
 					added = true;
 				}
-			} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-				addListItem(i.value(), name);
-				added = true;
 			}
 		}
-		if( iSelectedStatFilter == Package::PkgStatInstalled || iSelectedStatFilter == Package::PkgStatUnknown && !added)
+	} else {
+		for( int j=0; j<iSearchResults.count(); j++ )
 		{
-			if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter)) )
-			{
-				addListItem(i.value(), name);
-				added = true;
-			} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-				addListItem(i.value(), name);
-				added = true;
-			}
-		}
-		if( iSelectedStatFilter == Package::PkgStatUpgradeable || iSelectedStatFilter == Package::PkgStatUnknown && !added)
-		{
-			if( i.value()->isInstalled() && iSelectedCatFilter!=CatFilterAllMarked && i.value()->versionAvailable()!=i.value()->versionInstalled()
-				&& i.value()->versionAvailable()!="" && i.value()->section().startsWith(iCatFilterStrings.at(iSelectedCatFilter))
-				&& !i.value()->isLocal() )
-			{
-				if( iSelectedCatFilter==CatFilterAllUser && !i.value()->section().startsWith("user/hidden") ) {
-					addListItem(i.value(), name);
-					added = true;
-				}
-				else if( iSelectedCatFilter!=CatFilterAllUser ) {
-					addListItem(i.value(), name);
-					added = true;
-				}
-			} else if(iSelectedCatFilter==CatFilterAllMarked && i.value()->isMarkedForOperation() ) {
-				addListItem(i.value(), name);
-				added = true;
-			}
+			QString name = iSearchResults.at(j)->name();
+			if( iSearchResults.at(j)->maemoDisplayName()!="" )
+				name = iSearchResults.at(j)->maemoDisplayName();
+			QString n( name.at(0) );
+			n = n.toUpper();
+			name.replace(0,1,n);
+
+			addListItem(iSearchResults.at(j), name);
 		}
 	}
 
 	updateLabel();
+
 	show();
 
+	if( !ui->searchBar->isVisible() ) {
+		ui->listWidget->setFocusPolicy(Qt::StrongFocus);
+		ui->listWidget->setFocus();
+	} else {
+		ui->listWidget->setFocusPolicy(Qt::NoFocus);
+		ui->lineEdit->setFocus();
+	}
+
 	if( ui->listWidget->count() == 0 )
 	{
 		iListCoverLabel->setGeometry( ui->listWidget->rect() );
@@ -370,7 +402,6 @@ void PackageView::closeEvent(QCloseEvent *event)
 	if( iAptInterface->selectedCount() == 0 )
 	{		
 		resetWindow();
-		iMainWindow->setOperation(MainWindow::OpNone);
 		event->accept();
 	} else {
 		QString c;
@@ -379,7 +410,6 @@ void PackageView::closeEvent(QCloseEvent *event)
 		d.setText(tr("Returning to main menu"), tr("Clear ") + c + tr(" package selection(s) and lose all the pending changes?"));
 		if( d.exec() ) {
 			resetWindow();
-			iMainWindow->setOperation(MainWindow::OpNone);
 			event->accept();
 		} else {
 			event->ignore();
@@ -436,22 +466,21 @@ void PackageView::on_actionClear_selections_triggered()
 	d.setText(tr("Confirmation"), tr("Clear ") + c + tr(" package selection(s) and lose all the pending changes?"));
 	if( d.exec() )
 	{
-		QHashIterator<QString, Package*> i( *iAptInterface->packages() );
-		while (i.hasNext())
-		{
-			i.next();
-			i.value()->setMarkedForOperation(Package::PkgOpNone);
-		}
-
-		iAptInterface->resetSelectedCount();
-
+		clearSelections();
 		openWin(iOperation);
 	}
 }
 
-void PackageView::on_actionCategoryFilter_triggered()
+void PackageView::clearSelections()
 {
-	on_btn_CategoryFilter_clicked();
+	QHashIterator<QString, Package*> i( *iAptInterface->packages() );
+	while (i.hasNext())
+	{
+		i.next();
+		i.value()->setMarkedForOperation(Package::PkgOpNone);
+	}
+
+	iAptInterface->resetSelectedCount();
 }
 
 void PackageView::on_listWidget_itemClicked(QListWidgetItem* item)
@@ -496,9 +525,11 @@ void PackageView::on_btn_CategoryFilter_clicked()
 {
 	FilterSelect f("Category filter", this);
 	f.setList( iCatFilterLabels, iSelectedCatFilter );
-	if( f.exec() ) {
+
+	bool s = f.exec();
+
+	if( s )
 		iSelectedCatFilter = f.selection();
-	}
 
 	if( iSelectedCatFilter == CatFilterAllMarked ) {
 		ui->btn_StatusFilter->setEnabled(false);
@@ -506,7 +537,8 @@ void PackageView::on_btn_CategoryFilter_clicked()
 		ui->btn_StatusFilter->setEnabled(true);
 	}
 
-	openWin(iOperation);
+	if( s )
+		openWin(iOperation);
 }
 
 void PackageView::setStatFilter(Package::packageStatus f_)
@@ -514,26 +546,30 @@ void PackageView::setStatFilter(Package::packageStatus f_)
 	iSelectedStatFilter = f_;
 }
 
-void PackageView::on_actionStatus_filter_triggered()
-{
-	on_btn_StatusFilter_clicked();
-}
-
 void PackageView::on_btn_StatusFilter_clicked()
 {
 	FilterSelect f("Status filter", this);
 	f.setList( iStatFilterLabels, iSelectedStatFilter );
-	if( f.exec() ) {
-		iSelectedStatFilter = (Package::packageStatus)f.selection();
-	}
 
-	openWin(iOperation);
+	bool s = f.exec();
+
+	if( s ) {
+		iSelectedStatFilter = (Package::packageStatus)f.selection();
+		openWin(iOperation);
+	}
 }
 
 void PackageView::resetWindow()
 {
 	iSelectedCatFilter = iDefaultCatFilter;
 	ui->btn_StatusFilter->setEnabled(true);
+
+	releaseKeyboard();
+	clearSelections();
+	if( iMainWindow )
+		iMainWindow->setOperation(MainWindow::OpNone);
+
+	clearSearch();
 }
 
 void PackageView::on_actionHelp_triggered()
@@ -541,3 +577,84 @@ void PackageView::on_actionHelp_triggered()
 	Help h(this);
 	h.exec();
 }
+
+void PackageView::on_btn_searchClose_clicked()
+{
+	clearSearch();
+	openWin(iOperation);
+}
+
+void PackageView::clearSearch()
+{
+	ui->lineEdit->clear();
+	ui->searchBar->hide();
+	ui->toolBar->show();
+	ui->listWidget->setFocusPolicy(Qt::StrongFocus);
+
+	iSearchResults.clear();
+}
+
+void PackageView::on_actionSearch_triggered()
+{
+	if( ui->searchBar->isVisible() )
+		return;
+
+	ui->listWidget->setFocusPolicy(Qt::NoFocus);
+	ui->searchLabel->setText( QString("%0 results").arg(ui->listWidget->count()) );
+	ui->toolBar->hide();
+	ui->searchBar->show();
+	ui->lineEdit->setFocus();
+	iPrevSearchText = "";
+}
+
+void PackageView::on_lineEdit_textEdited(QString text)
+{
+	if( text=="" ) {
+		on_btn_searchClose_clicked();
+		return;
+	}
+
+	if( iPrevSearchText.length() > text.length() )
+	{
+		ui->searchBar->hide();
+		openWin(iOperation);
+		ui->searchBar->show();
+		ui->lineEdit->setFocus();
+	}
+
+	iPrevSearchText = text;
+	QList<QListWidgetItem*> results = ui->listWidget->findItems( text, Qt::MatchContains );
+
+	ui->searchLabel->setText( QString("%0 results").arg(results.count()) );
+
+	iSearchResults.clear();
+	for( int i=0; i<results.count(); i++ )
+	{
+		iSearchResults.append( static_cast<PackageListWidgetItem*>(results.at(i))->package() );
+	}
+	openWin(iOperation);
+}
+
+void PackageView::setSearchText(QString text)
+{
+	ui->lineEdit->setText(text);
+	on_lineEdit_textEdited(text);
+}
+
+KeyEventGrabber::KeyEventGrabber(QObject* parent) : QObject(parent)
+{
+}
+
+bool KeyEventGrabber::eventFilter(QObject *obj, QEvent *event)
+{
+	if( event->type() == QEvent::KeyPress ) {
+		QString text = static_cast<QKeyEvent*>(event)->text();
+		int key = static_cast<QKeyEvent*>(event)->key();
+		if( (text.trimmed() != "" || text==" ") && key!=Qt::Key_Backspace ) {
+			dynamic_cast<PackageView*>(this->parent())->on_actionSearch_triggered();
+			dynamic_cast<PackageView*>(this->parent())->setSearchText( text );
+			return true;
+		}
+	}
+	return QObject::eventFilter(obj, event);
+}
