@@ -33,7 +33,7 @@ class MainWindow;
 class AptInterface;
 class Package;
 class dimmer;
-
+class KeyEventGrabber;
 
 enum UserRoles { UserRoleName=Qt::UserRole, UserRoleVerInst, UserRoleVerAvail, UserRoleDescShort, UserRoleOperation,
 				 UserRoleMarked, UserRoleInstalled, UserRoleUpgradeable };
@@ -85,7 +85,8 @@ public:
 	void disableMenu();
 	void enableMenu();
 	void setStatFilter(Package::packageStatus f_);
-	void resetWindow();
+	void clearSelections();
+	void setSearchText(QString text);
 
 #ifdef Q_WS_MAEMO_5		// workaround for buggy Qt autorotation
 	void show() {
@@ -100,6 +101,7 @@ public:
 
 public slots:
 	void orientationChanged();
+	void on_actionSearch_triggered();
 
 protected:
     void changeEvent(QEvent *e);
@@ -110,6 +112,8 @@ protected:
 private:
 	void addListItem(Package* pkg_, QString listname_);
 	void updateLabel(void);
+	void resetWindow();
+	void clearSearch();
 
     Ui::PackageView *ui;
 	MainWindow* iMainWindow;
@@ -127,15 +131,32 @@ private:
 	QStringList iStatFilterLabels;
 	int iSelectedStatFilter;
 
+	QList<Package*> iSearchResults;
+	QString iPrevSearchText;
+	KeyEventGrabber* iKeyFilter;
+
 private slots:
+	void on_lineEdit_textEdited(QString );
+	void on_btn_searchClose_clicked();
 	void on_actionHelp_triggered();
- void on_btn_StatusFilter_clicked();
-	void on_actionStatus_filter_triggered();
+	void on_btn_StatusFilter_clicked();
 	void on_btn_CategoryFilter_clicked();
 	void on_listWidget_itemClicked(QListWidgetItem* item);
-	void on_actionCategoryFilter_triggered();
 	void on_actionClear_selections_triggered();
 	void on_btn_Commit_clicked();
 };
 
+
+class KeyEventGrabber : public QObject
+{
+	Q_OBJECT
+public:
+	KeyEventGrabber(QObject* parent = 0);
+
+protected:
+	bool eventFilter(QObject *obj, QEvent *event);
+
+};
+
+
 #endif // PACKAGEVIEW_H
